package com.caimei365.order.components;

import com.caimei365.order.mapper.BaseMapper;
import com.caimei365.order.model.vo.CartItemVo;
import com.caimei365.order.model.vo.LadderPriceVo;
import com.caimei365.order.model.vo.PromotionPriceVo;
import com.caimei365.order.model.vo.PromotionsVo;
import com.caimei365.order.utils.MathUtil;
import com.caimei365.order.utils.ProductUtil;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang.ArrayUtils;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;
import org.springframework.util.CollectionUtils;

import javax.annotation.Resource;
import java.math.BigDecimal;
import java.util.ArrayList;
import java.util.List;
import java.util.stream.IntStream;

/**
 * Description
 *
 * @author : Charles
 * @date : 2021/6/29
 */
@Slf4j
@Service
public class ProductService {
    @Value("${caimei.wwwDomain}")
    private String domain;
    @Resource
    private BaseMapper baseMapper;

    /**
     * 设置商品图片及税费
     * @param cartItemVo 购物车商品
     * @return taxFlag 是否需要设置税费
     */
    public boolean setCartItemImgAndTax(CartItemVo cartItemVo) {
        // 图片路径
        String image = ProductUtil.getImageUrl("product", cartItemVo.getImage(), domain);
        cartItemVo.setImage(image);
        // 是否添加税费，不含税商品 开票需添加税费
        boolean taxFlag = "0".equals(cartItemVo.getIncludedTax()) && ("1".equals(cartItemVo.getInvoiceType()) || "2".equals(cartItemVo.getInvoiceType()));
        if (taxFlag) {
            BigDecimal cartItemTax = MathUtil.div(MathUtil.mul(cartItemVo.getPrice(), cartItemVo.getTaxRate()), 100, 2);
            cartItemVo.setPrice(MathUtil.add(cartItemVo.getPrice(), cartItemTax).doubleValue());
        }
        return taxFlag;
    }

    /**
     * 设置购物车阶梯价
     * @param cartItemVo
     * @param taxFlag
     */
    public void setCartLadderPrices(CartItemVo cartItemVo, boolean taxFlag) {
        // 阶梯价列表
        List<LadderPriceVo> ladderPrices = baseMapper.getLadderPriceList(cartItemVo.getProductId());
        if (!CollectionUtils.isEmpty(ladderPrices)) {
            IntStream.range(0, ladderPrices.size()).forEach(i -> {
                boolean isThisLadder;
                // 添加税费
                if (taxFlag) {
                    BigDecimal addedValueTax = MathUtil.div(MathUtil.mul(ladderPrices.get(i).getBuyPrice(), cartItemVo.getTaxRate()), 100, 2);
                    ladderPrices.get(i).setBuyPrice(MathUtil.add(ladderPrices.get(i).getBuyPrice(), addedValueTax).doubleValue());
                }
                if (i == ladderPrices.size() - 1) {
                    ladderPrices.get(i).setMaxNum(0);
                    ladderPrices.get(i).setNumRange("≥" + ladderPrices.get(i).getBuyNum());
                    isThisLadder = (cartItemVo.getNumber() >= ladderPrices.get(i).getBuyNum());
                } else {
                    ladderPrices.get(i).setMaxNum(ladderPrices.get(i + 1).getBuyNum());
                    String buyNumRangeShow = ladderPrices.get(i).getBuyNum() + "~" + (ladderPrices.get(i + 1).getBuyNum() - 1);
                    ladderPrices.get(i).setNumRange(buyNumRangeShow);
                    isThisLadder = (cartItemVo.getNumber() >= ladderPrices.get(i).getBuyNum() && cartItemVo.getNumber() <= ladderPrices.get(i).getMaxNum());
                }
                if (isThisLadder) {
                    cartItemVo.setPrice(ladderPrices.get(i).getBuyPrice());
                    cartItemVo.setOriginalPrice(ladderPrices.get(i).getBuyPrice());
                }
            });
            cartItemVo.setMin(ladderPrices.get(0).getBuyNum());
            cartItemVo.setLadderPrices(ladderPrices);
        } else {
            cartItemVo.setLadderFlag(0);
        }
    }

    /**
     * 当前促销活动的价格计算列表
     *
     * @param promotions 促销活动
     * @param cartItemVo 当前购物车商品
     * @param taxFlag    计算税费标志
     * @return 价格计算列表
     */
    public List<PromotionPriceVo> getPromotionProducts(PromotionsVo promotions, CartItemVo cartItemVo, boolean taxFlag) {
        List<PromotionPriceVo> promotionPriceList = new ArrayList<>();
        PromotionPriceVo promotionPrice = new PromotionPriceVo();
        promotionPrice.setProductId(cartItemVo.getProductId());
        promotionPrice.setNumber(cartItemVo.getNumber());
        // 设置价格
        if (promotions.getType() == 1 && promotions.getMode() == 1) {
            // 单品优惠价添加税费
            if (taxFlag) {
                BigDecimal addedValueTax = MathUtil.div(MathUtil.mul(promotions.getTouchPrice(), cartItemVo.getTaxRate()), BigDecimal.valueOf(100), 2);
                promotions.setTouchPrice(MathUtil.add(promotions.getTouchPrice(), addedValueTax).doubleValue());
            }
            promotionPrice.setPrice(promotions.getTouchPrice());
            promotionPrice.setOriginalPrice(cartItemVo.getOriginalPrice());
            // 设置商品活动价
            cartItemVo.setPrice(promotions.getTouchPrice());
        } else {
            promotionPrice.setPrice(cartItemVo.getPrice());
        }
        promotionPriceList.add(promotionPrice);
        return promotionPriceList;
    }

    /**
     * 把当前促销更新购物车总促销
     * @param totalPromotions 总促销列表
     * @param promotionsIds   总促销Id集合
     * @param promotions      当前促销
     * @param promotionPriceList  当前促销商品价格计算列表
     */
    public void updateTotalPromotions(List<PromotionsVo> totalPromotions, List<Integer> promotionsIds, PromotionsVo promotions, List<PromotionPriceVo> promotionPriceList) {
        if (promotionsIds.contains(promotions.getId())) {
            // 列表已有该促销活动
            totalPromotions.forEach(item -> {
                if (item.getId().equals(promotions.getId())) {
                    promotionPriceList.addAll(item.getProductList());
                    item.setProductList(promotionPriceList);
                }
            });
        } else {
            // 新的促销活动
            promotionsIds.add(promotions.getId());
            if (promotions.getMode() == 3) {
                // 获取赠品
                List<CartItemVo> giftList = baseMapper.getPromotionGifts(promotions.getId());
                promotions.setGiftList(giftList);
            }
            promotions.setProductList(promotionPriceList);
            // 添加到总促销
            totalPromotions.add(promotions);
        }
    }

    /**
     * 是否订单充值商品或定金商品
     * @param productId 商品id
     * @return Boolean
     */
    public boolean isRechargeProduct(Integer productId){
        int[] products = {6060, 6061, 6062, 6063, 6064, 6065, 6066, 6067, 6068, 6069};
        return ArrayUtils.contains(products, productId);
    }

}
